#ifndef RASTER_UTIL_H_
#define RASTER_UTIL_H_

//
// misc raster utilities and defines
// internal usage only.
//






// 
// informational warnings if your warning level is set 
// to level 4 in MSVC.
//
#pragma warning( disable : 4512 4244 4514 )


//
// useful types
//
typedef unsigned char  byte;
typedef unsigned long  u32;
typedef unsigned short u16;




//
// returns the square of a number
// using a template avoid's evaluating 'N' twice. 
template <class T>  inline T square(T N)  { return N*N; }

// swaps two objects
template <class T> inline void swap_obs(T & a, T & b ) { T tmp = a; a = b; b = tmp; }  







// returns  (1<<exp)
inline float fshl( int exp)    { int rv = ( (127+(exp)) << 23); return  (float&)rv; }  
// returns  (1<<exp) + (1<< (exp-1))
inline float fshl_hf(int exp)  { int rv = ( (127+(exp)) << 23) + (1<<22); return  (float&)rv; }  


//
// float -> int  conversions
//
extern "C" double misc_qw_store0;
extern "C" float  d_to_int_magic;
extern "C" float  d_to_fx16_magic;


//
// Note: for these to work, the fp unit must be in double precision mode.
//
// converts a float/double to an int
inline int d_to_int(double d)
{
 misc_qw_store0 = d + d_to_int_magic;
 return (int&)misc_qw_store0;
}
// same but adds mults but 65536
inline int d_to_fx16(double d)
{
 misc_qw_store0 = d + d_to_fx16_magic;
 return (int&)misc_qw_store0;
}
//
// float to int signed.
// works for numbers in the range (1<<21)-1  .. -(1<<21)
// the fp unit need not be in double precision mode.
//

inline int f_to_int_s(float f, int exp_scale = 0)
{
 int rv;
 (float&)rv = f + fshl_hf( 23 - exp_scale);
 return (rv<< (32-22) ) >> (32-22);
}






//
//
//  Fixed point constants
//
//

template <int BITS>
struct fixed_const
 {
   // float version of 1 in fixed point
   static float r() { return (float)(1<<BITS); }

   // int version of 1 in fixed point
   static int  i() { return (1<<BITS); }

   // eg.  a&frac_mask == fractional component of a
   static int  frac_mask() { return i() - 1; }

   // eg.  a&whole_mask == whole  component of a
   static int  whole_mask() { return ~frac_mask(); }

   // get fractional part of a
   static int  frac(int a) { return a & frac_mask(); }

   // get normalized fraction of a (eg in the range 0.0 <= nfrac < 1.0)
   static float  nfrac(int a) { return frac(a)/r(); }

   // number of bits
   static int  b() { return  BITS; }

   static int  round_up(int a)   {  return   (a+frac_mask() )& whole_mask() ; }
   static int  round_down(int a) {  return    a&whole_mask(); }

   // divide an integer
   // important: MSVC will insert a cdq to round towards 0 if you use a /
   static int   d(int a )  { return a >> BITS; }

   // divide and integer, rounding up
   static int   du(int a )  { return (a+frac_mask()) >> BITS; }

 };
extern fixed_const<16> FX1;
#endif